@@ -43,6 +43,7 @@
             print_html_head/1,          % +Stream
             predref//1,                 % +PI //
             predref//2,                 % +PI, Options //
+            nopredref//1,               % +PI //
             module_info/3,              % +File, +Options0, -Options
             doc_hide_private/3,         % +Doc0, -Doc, +Options
             edit_button//2,             % +File, +Options, //
@@ -71,6 +72,7 @@
             tags//1,                    % +Tags, //
             term//3,                    % +Text, +Term, +Bindings, //
             file_header//2,             % +File, +Options, //
+            flagref//1,                 % +Flag
             objects//2,                 % +Objects, +Options, //
             object_ref//2,              % +Object, +Options, //
             object_name//2,             % +Object, +Object
@@ -79,7 +81,9 @@
             object_page//2,             % +Object, +Options, //
             object_page_header//2,      % +File, +Options, //
             object_synopsis//2,         % +Object, +Options, //
-            object_page_footer//2       % +Object, +Options, //
+            object_footer//2,           % +Object, +Options, //
+            object_page_footer//2,      % +Object, +Options, //
+            cite//1                     % +Citations
           ]).
 :- use_module(library(lists)).
 :- use_module(library(option)).
@@ -508,6 +512,8 @@ private(Object, _Options):-
     prolog:doc_is_public_object(Object), !, fail.
 private(Module:PI, Options) :-
     multifile(Module:PI, Options), !, fail.
+private(Module:PI, Options) :-
+    public(Module:PI, Options), !, fail.
 private(Module:PI, Options) :-
     option(module(Module), Options),
     option(public(Public), Options),
@@ -541,7 +547,20 @@ multifile(Obj, _Options) :-
     pi_to_head(PI, Head),
     (   predicate_property(Module:Head, multifile)
     ;   xref_module(Source, Module),
-        xref_defined(Source, Head, multifile(_))
+        xref_defined(Source, Head, multifile(_Line))
+    ),
+    !.
+
+%!  public(+Options, +Options)
+%
+%   True if Obj is declared using public/1.
+
+public(Obj, _Options) :-
+    strip_module(user:Obj, Module, PI),
+    pi_to_head(PI, Head),
+    (   predicate_property(Module:Head, public)
+    ;   xref_module(Source, Module),
+        xref_defined(Source, Head, public(_Line))
     ),
     !.
 
@@ -786,7 +805,9 @@ pred_dom(Obj, Options, Pos-Comment, DOM) :-
             )
         ;   Class = multidef(file((Pos)))
         )
-    ;   Class = pubdef              % public definition
+    ;   public(Obj, Options)
+    ->  Class = publicdef           % :- public definition
+    ;   Class = pubdef              % exported definition
     ),
     (   Obj = Module:_
     ->  POptions = [module(Module)|Options]
@@ -1020,13 +1041,25 @@ file_link(File) -->
     html([ div(a(href(location_by_id(pldoc_doc)+File), File))
          ]).
 
+%!  object_footer(+Obj, +Options)// is det.
+%
+%   Call the hook prolog:doc_object_footer//2. This hook will be used to
+%   deal with examples.
+
+object_footer(Obj, Options) -->
+    prolog:doc_object_footer(Obj, Options),
+    !.
+object_footer(_, _) --> [].
+
+
 %!  object_page_footer(+Obj, +Options)// is det.
 %
 %   Call the hook prolog:doc_object_page_footer//2. This hook will
 %   be used to deal with annotations.
 
 object_page_footer(Obj, Options) -->
-    prolog:doc_object_page_footer(Obj, Options).
+    prolog:doc_object_page_footer(Obj, Options),
+    !.
 object_page_footer(_, _) --> [].
 
 
